//-------------------------------------
// describing one box
// element and index in box sequence

// bs 20150111
//--------------------------------------
function itemBox(){
	// elements
	var wrapper=undefined;
	var index=undefined;
	var elementId=undefined;
	var header=undefined;
	var toggler=undefined;
	var content=undefined;
	var wrapperOriginal=undefined;

	// options as in classList
	var toBeDragged=undefined;
	var toBeToggled=undefined;
	var initiallyOn=undefined;
	var toBeRemembered=undefined;
	var noHeader=undefined;
	var titleTextNode=undefined;
	var ownStyle=undefined;
	var kill=undefined;
	
	// dragging
	var initialMouseX,initialMouseY,startX,startY;
	var mouseMoveHandler,mouseUpHandler;

	
	//*********************************************************
	//******************  exposed API **************************
    //-----------  getters/setters -------------------	
	// id
	this.getId=function(){return elementId; }
	// index
	this.getIndex=function(){return index; }	
	// title
	this.getTitle=function(){return titleTextNode.textContent;}
	this.setTitle=function(t){titleTextNode.textContent=t;}	
	// kill, is it set to be killable according to initial option
	this.getKill=function(){return kill;}	
	// has header according to initial option "no-header"
	// not updated when hiding header
	this.hasHeader=function(){return !noHeader;}
	// content as innerHTML
	this.getContentHTML=function(){return content.innerHTML;},	
    this.setContentHTML=function(t){content.innerHTML=t;},	
	// content as element
	this.getContentElement=function(){return content;},
	this.setContentElement=function(newElt){
		// remove everything within content
		var elt=content.firstChild;
		while(elt!=null){
			elt=elt.nextSibling;
			if(elt!=null)
				content.removeChild(elt);
		}			
		content.appendChild(newElt.cloneNode(true));
	}	
	// wrapper element
	this.getWrapperElement=function(){return wrapper;}		
	// get current position
	this.getCurrentPos=function(){
		// no left,top ??
		if((wrapper.style.left=='')||(wrapper.style.top==''))
			return itemHolder.getPosition(wrapper);
		return{left:parseInt(wrapper.style.left),
		       top:parseInt(wrapper.style.top)};
	}
	// set current position (Left,Top)
	// (this is an issue only before any movements have occured)
	this.setCurrentPos=function(L,T){
		// kill right and bottom				
		var s=wrapper.getAttribute("style");
		if(s==null || s==""){
			wrapper.setAttribute("style",'left:'+L+'px;top:'+T+'px');
		}
		else{
			wrapper.style.right="";
			wrapper.style.bottom="";
			wrapper.style.left=parseInt(L)+"px";
			wrapper.style.top=parseInt(T)+"px";
		}
	}	
	// z index
	this.getZIndex=function(){
		return wrapper.style.zIndex;
	}
	// also used by itemHolder:
	this.setZIndex=function(z){
		wrapper.style.zIndex=z;
	}
	// --------------------------------------------
	// remove header and make content draggable
	// except those in the comma-separated list in parameter liststring
	// button is default
	this.setDragByContent=function(liststring){
		if(!liststring)
			liststring="button";
		var names=liststring.split(',');
		
		if(content.childNodes.length==0){
			content.innerHTML="<p>nothing</p>";
		}
		// make header invisible
		header.style.display="none";
		// give content drag-class and make sure it is displayed
		content.classList.add("drag");
		this.showContent();//this.content.style.display="block";
		// make it draggable (same way as we make header so)
		if(!content._alreadyListening)
				content.addEventListener ('mousedown',this.startDragMouse.bind(this),false);
		content._alreadyListening=true;


		// let all elements in content pass events
		// except those set in parameter (button is default)
		p=content.firstChild;
		while(p !=null){
			if(names.indexOf(p.nodeName)==-1){
				p.onmousedown=function(e){return false};
				p.onmousemove=function(e){return false};
			}
			p=p.nextSibling;
		}
	}

	//------------------------------------------
	//-------------- eventpickers ---------------
	// may be redefind to perform some action when this happens
	// called when dragstart and drag stop
	this.onStartDrag=function(){}
	this.onEndDrag=function(){}	
	// The result of a drag, ended at L,T
	// called before onEndDrag
	//this.hasMoved=function(L,T){}
	// called when toggled
	this.onShowContent=function(){}
	this.onHideContent=function(){}	
	// called when save
	this.onSave=function(){}
	
		
	//------------------------------------------
	// toggling content
	
	this.toggle=function(e){
		var evt = e || window.event;
		evt.stopPropagation();
		if(toggler.classList.contains("off"))
			this.toggleOn();
		else
			this.toggleOff();
	}
	//------------------------------------------
	
	//***************************************************
	//------------ status changers ---------------
	// show/hide complete box
	this.showBox=function(){wrapper.style.display="block";}
	this.hideBox=function(){wrapper.style.display="none";}
	
	// show/hide content
	this.showContent=function(){content.style.display="block";}
	this.hideContent=function(){content.style.display="none";}
	
	// show/hide header
	this.showHeader=function(e){
		if(this.hasHeader())
			header.style.display="block";
		if(e)e.stopPropagation();
	}
	this.hideHeader=function(e){
		header.style.display="none";
		if(e)e.stopPropagation();
	}	

	//------------------------
	// toggle functionality
	this.showToggler=function(e){
		toggler.style.display="block";
		if(e)e.stopPropagation();
	}
	this.hideToggler=function(e){
		toggler.style.display="none";
		if(e)e.stopPropagation();
	}
	this.toggleOn=function(){
			this.onShowContent(); 	//!!
			this.showContent();
			toggler.classList.remove("off");
			toggler.classList.add("on");
			this.moveToFront();
	}
	this.toggleOff=function(){
			this.onHideContent();	//!!
			if(kill){
				this.hideBox();
				// prepare for off if it is initiated again
				toggler.classList.remove("off");
				toggler.classList.add("on");
				this.showContent();
			}
			else{
				// normal case
				this.hideContent();
				toggler.classList.remove("on");
				toggler.classList.add("off");
			}
	}

	//-------------------
	// drag functionality	
	this.turnOffDrag=function(){
		if(!this.hasHeader()){
			// turn off content drag
			content.classList.add("nodrag");
			content.classList.remove("drag");
			// make it draggable (same way as we make header so
			content.onmousedown = null;
		}
		else{
			header.classList.add("nodrag");
			header.classList.remove("drag");
			header.onmousedown = null;
		}
	}
	this.turnOnDrag=function(){
		if(!this.hasHeader()){
			// turn on content drag
			this.setDragByContent()
		}
		else{
			header.classList.add("drag");
			header.classList.remove("nodrag");
			header.addEventListener ('mousedown',this.startDragMouse.bind(this),false);

		}
	}
	
	//--------------------------------
	// move it on the z axis to front
	this.moveToFront=function(){
		var max=itemHolder.getMaxZIndex();
		this.setZIndex(max+1);
		itemHolder.setMaxZIndex(max+1);
	}
	//-----------------------------------------
	//***********  eof exposed API ************************
	//-----------------------------------------
	//***********  dragging ************************
	
	this.startDragMouse = function (e) {
		var evt = e || window.event;
		//console.log(evt);
		evt.stopPropagation();
		// if we get it from toggler, forget it
		if(evt.target == toggler){
			return;
		}
		// this is header or content
		startX = wrapper.offsetLeft;
		startY = wrapper.offsetTop;

		wrapper.classList.add('dragged');
		itemHolder.incrementMaxZIndex();
		wrapper.style.zIndex=itemHolder.getMaxZIndex();
		
		// give us a chance to program on startDrag
		//this.onStartDrag();

		// want to identify the boxobject, by its wrapper
		//itemHolder.draggedBox=itemHolder.getBoxByElement(this.parentNode);

		initialMouseX = evt.clientX;
		initialMouseY = evt.clientY;
		
		mouseMoveHandler=this.dragMouse.bind();
		mouseUpHandler=this.releaseElement.bind();
		

		document.addEventListener ('mouseup',mouseUpHandler,false);
		document.addEventListener ('mousemove',mouseMoveHandler,false);

		
		//return false;
	}

	
	this.dragMouse= function (e) {
		var evt = e || window.event;
		var dX = evt.clientX - initialMouseX;
		var dY = evt.clientY - initialMouseY;
		wrapper.style.left =
			Math.min( Math.max(startX + dX,0),document.documentElement.clientWidth-20) + 'px';
		wrapper.style.top = 
			Math.min(Math.max(startY + dY,0),document.documentElement.clientHeight-20) + 'px';
		
		// kill right and bottom 
		wrapper.style.right='';
		wrapper.style.bottom='';

		//e.stopPropagation();
		//return false;
	}
	
	this.releaseElement=function(e) {
		//console.log("release",mouseMoveHandler);

		document.removeEventListener('mouseup',mouseUpHandler,false);
		document.removeEventListener('mousemove',mouseMoveHandler,false);

		wrapper.classList.remove('dragged');
		//e.stopPropagation();
		//------------
		// give user a chance to react
		itemHolder.getBoxByElement(wrapper).onEndDrag();			

		//----------------

		//return false;
	}
	
	
	
	//******************************************
	//-------------- persistance ---------------
	this.getKey=function(){
		var key="itemBox_"+elementId;
		if(document.title)
			key=document.title+'_'+key;
		return key;
	}
	
	this.save=function(){
		var key=this.getKey();
		if(toBeRemembered){
			// main wrapper style
			var style=wrapper.getAttribute("style");
			// content style
			var contentStyle=content.getAttribute("style");
			// togglespan style
			var spanClass=toggler.getAttribute("class");
			var jsObj={id:elementId,
			           wrapperStyle:style,
			           contentStyle:contentStyle,
					   spanclass:spanClass};
			var jsString=JSON.stringify(jsObj);
			localStorage.setItem(key,jsString);
		}
		else
			localStorage.removeItem(key);
		// anything else we want to remember in this box
		this.onSave();
	}
	
	this.restore=function(){
		var key=this.getKey();
		var jsString=localStorage.getItem(key);
		if(jsString){
			var jsObj=JSON.parse(jsString);
			wrapper.setAttribute("style",jsObj.wrapperStyle);
			content.setAttribute("style",jsObj.contentStyle);
			toggler.setAttribute("class",jsObj.spanclass);
		}
		// has no meaning ?:
		//this.onRestore();
	}	
	//----------------------------------------------
	
	//*************************************************
	//------------------ setup--------------------------
	this.setUp=function(e,i){
		// set up everything
		// and collect them as need at the end
		wrapper=e;
		index=i;

	
		elementId=wrapper.getAttribute("id");
		header=document.createElement("div");
		toggler=document.createElement("span");// the on/off trigger
		content=document.createElement("div");
	
		// title in a text node
		var title=wrapper.getAttribute("title");
		if(!title)
			title=elementId;		
		titleTextNode=document.createTextNode(title);

		// pick up parameters (classnames) relevant to itemHolder
		toBeDragged=wrapper.classList.contains("drag");
		toBeToggled=wrapper.classList.contains("expand");
		initiallyOn=wrapper.classList.contains("on");
		toBeRemembered=wrapper.classList.contains("remember");
		kill=wrapper.classList.contains("kill");
		noHeader=wrapper.classList.contains("no-header");

			
		if(!toBeToggled)
			initiallyOn=true;
			
		
		// find special style if any (style-XXX)
		for(var ix=0;ix < wrapper.classList.length;ix++)
		{
			var t=wrapper.classList[ix];
			if(t.substring(0,6)=="style-"){
				ownStyle=t.substring(6);
				break;
			}
		}

		// remove itemHolder relevant classes, parameters
		// (keep itembox)
		wrapper.classList.remove("drag");
		wrapper.classList.remove("expand");
		wrapper.classList.remove("on");
		wrapper.classList.remove("off");// to make sure

		// move others to content, except itembox
		for(ix=wrapper.classList.length-1;ix > 0;ix--)
			if(wrapper.classList[ix]!="itembox"){
				content.classList.add(wrapper.classList[ix]);
				wrapper.classList.remove(wrapper.classList[ix]);
		}
		// if own style we replace original and itembox with ownStyle 
		if(ownStyle){
			wrapper.classList.remove("style-"+ownStyle);
			wrapper.classList.remove("itembox");
			wrapper.classList.add(ownStyle);
			// and clean up content
			content.classList.remove("style-"+ownStyle);
			content.classList.add("content");
		}
		
		// mark it to avoid double initialization
		wrapper.classList.add("readybox");


		//make sure content is reachable by  original id
		content.setAttribute("id",elementId);
		// set wrapper id on original element
		wrapper.setAttribute("id",elementId+"Wrapper");
		content.classList.add("content");

		// preserve original content(if any) in content
		content.innerHTML=wrapper.innerHTML;
		// and remove it from wrapper
		wrapper.innerHTML="";

		// prepare clickable toggler element
		// this has a function: toggle
		toggler.addEventListener('click',this.toggle.bind(this),false);
		toggler.classList.add("toggler");

		
		if(initiallyOn)
			toggler.classList.add("on");
		else
			toggler.classList.add("off");

		// prepare header that may be draggable (drag)
		header.classList.add("header");
		if(toBeDragged){
			header.classList.add("drag");
			header.addEventListener ('mousedown',this.startDragMouse.bind(this),false);

		}
		else{
			header.classList.add("nodrag");
			header.addEventListener ('mousedown',this.moveToFront.bind(this),false);
		}
		
		this.setInitialPosition();
		this.configure();
		
	}

	//------------------------------------
	// initial positioning
	this.setInitialPosition=function(){

		// pick up the position-value given
		var eStyleposition=wrapper.style.position;	

		// pick up the top, left if given
		var top=wrapper.style.top;
		var left=wrapper.style.left;
		if(top=='') top='0';
		if(left=='')	left='0';
		top=parseInt(top);
		left=parseInt(left);

		if(eStyleposition=='relative'){
			// we want it absolute to simplify dragging				
			// calculate pos depending of anchestor
			pos=itemHolder.getPosition(wrapper);

			
			var newTop=pos.top-top;
			var newLeft=pos.left-left;				
			// look only one level up
			if(wrapper.parentNode.style.position == "absolute"){
				newTop=top;
				newLeft=left;
			}
			// set it
			wrapper.style.top=newTop+"px";
			wrapper.style.left=newLeft+"px";				

			//wrapper.style.position="absolute";
			// loose right and bottom
			wrapper.style.bottom="";
			wrapper.style.right="";

		}
		else if(eStyleposition=='fixed'){
			pos=itemHolder.getPosition(wrapper);
		}
		else if(eStyleposition==''){
			// no positioning, make absolute
			// assume top and left is ok or non-existent
			if(toBeDragged)
				wrapper.style.position="absolute";				
			// loose right and bottom
			wrapper.style.bottom="";
			wrapper.style.right="";
		}		
		else{
			//dont know what else to do
			// but set absolute
			// set left and top and loose right and bottom
			// leave position			
			pos=itemHolder.getPosition(wrapper);
			wrapper.style.top=pos.top+"px";
			wrapper.style.left=pos.left+"px";
			// loose right and bottom
			wrapper.style.bottom="";
			wrapper.style.right="";
		}
	}

	//--------------------------------
	// put it all together			
	this.configure=function(){
		
		// set correct style to content according to visibility
		// no other style values are copied from master element
		if(initiallyOn)
			content.setAttribute("style","display:block");
		else
			content.setAttribute("style","display:none");

		header.appendChild(titleTextNode);
		
		wrapper.appendChild(header);			
		if(toBeToggled)
			header.appendChild(toggler);
		wrapper.appendChild(content);
		
		if(noHeader){
			this.hideHeader();
			if(toBeDragged)
				this.setDragByContent();
				
		}

		// adjust itemHolder so it allways know top z
		if (wrapper.style.zIndex)
		{
			var thisIndex=wrapper.style.zIndex;
			if (thisIndex > itemHolder.MAX_Z_INDEX)
				itemHolder.MAX_Z_INDEX=thisIndex;
		}

		// set the correct with of this element (wrapper element)
		//console.log(elementId,wrapper.style.width,content.style.width);
		if(wrapper.style.width=='')
			wrapper.style.width=content.style.width;
		// all is set, remember the initial style of the wrapper
		// not sure why, not used
		wrapperOriginal=wrapper.cloneNode(false);// true ?		
	}
	
	//************************************************************
	//---------------------
	// while debugging
	this.dump=function(){
		console.log(this);
	}
	
}