//-----------------------------------------------
// library for handling popup boxes, 
// based on itemHolder and itemBox
// typical classlists for popboxes:
// 		itembox expand drag on style-popbox
// 		itembox expand drag kill  style-popbox on
// NOTE kill which leaves a box open until explicitly closed
// popbox-style is up to you, sample exists as popbox.css
// target for a pop is marked typically as
// 		<span id="pop1" title="box1" class="poptrigger">word </span>
// title identifies the box
// poptrigger is necessary to identify all popcases 
// when scanning the page
//
// NOTE that if box's title is set as "?" the box as popupbox will
// try to use triggers innerText, if not too long
// bs 20150111
//----------------------------------------------------
var popManager={
	//-------------------------------------
	// matching pop-elements and boxes
	// can also be made by hand:
	// popMatches=[{popId:"pop1",boxId:"box1"},{popId:"pop2",boxId:"box2"}]
	popMatches:undefined,
	// handling delay
	delay:300, // ms
	timer:null,

	//***************  exposed API ************
	// set delay for popup in ms
	setDelay:function(d){
		popManager.delay=parseInt(d);
	},
	//***************  eof exposed API ************	

	//-----------------------------------
	// scan page and find all pop-box pairs, popMatches
	// look for ".poptrigger"
	identifyPops:function (){	
		//var poppers=document.querySelectorAll(".poptrigger");
		var poppers=document.getElementsByClassName("poptrigger");
		for(var i=0;i <poppers.length;i++){
			// title is removed once registered
			if(poppers[i].hasAttribute("id") && poppers[i].hasAttribute("title")){			
				var popvalue=poppers[i].getAttribute("id");
				var boxvalue=poppers[i].getAttribute("title");
				popManager.popMatches.push({popId:popvalue,boxId:boxvalue});
			}
		}
	},

	//--------------------------------------
	// which box should we show
	findMatch:function (p){
		for(var i=0;i< popManager.popMatches.length;i++)
			if(popManager.popMatches[i].popId==p)
				return popManager.popMatches[i].boxId;
		return null;
	},

	//----------------------------------
	// prepare pop boxes, either simple or 
	// box with complete hide on toggle off
	setUpPop:function (){
		//popManager.setDelay(1000);
		// scan page if if so ordered
		if(!popManager.popMatches){
			popManager.popMatches=[];
			popManager.identifyPops();
		}
		for(var ix=0;ix< popManager.popMatches.length;ix++)
		{
			var elt=document.getElementById(popManager.popMatches[ix].popId);
			var box=itemHolder.getBoxById(popManager.popMatches[ix].boxId);

			elt.addEventListener('mouseenter',popManager.kickBox,false);
			elt.addEventListener('mouseleave',popManager.hideBox,true);
			
			// if not manual kill
			if(!box.getKill()){				 
				// dont show toggler, even if it is unreachable
				box.hideToggler();
			}
			
			// change header
			var T=elt.textContent;
			if(box.getTitle()=="?"){
				if(T.length < 25){
					box.setTitle(elt.textContent);
				}
				else{
					box.setTitle("");
				}
			}
			
			// mark trigger
			if(!elt.classList.contains("poptrigger"))
				elt.classList.add("poptrigger");
			elt.removeAttribute("title");
		}
	},

	//---------------------------------------------------
	// Initiate a timer to show box after delay
	kickBox(e){
		if(popManager.timer!=null){
			clearTimeout(popManager.timer);
			popManager.timer=null;
		}
		popManager.timer=setTimeout(function(){
			popManager.showBox(e.target) 
		}, popManager.delay);
	},
	
	//-----------------------------------------------
	// show the box
	showBox:function (target){
		if(popManager.timer!=null){
			clearTimeout(popManager.timer);
			popManager.timer=null;
		}
		//var target=e.target;
		var tpos=itemHolder.getPosition(target);
		//console.log(tpos);
		var boxid=popManager.findMatch(target.id);
		if(boxid!=null)
		{
			box=itemHolder.getBoxById(boxid);
			var wrapper=box.getWrapperElement();
			// where to put it
			var vW = Math.max(document.documentElement.clientWidth, window.innerWidth || 0)
			var vH = Math.max(document.documentElement.clientHeight, window.innerHeight || 0)
			// pop width/height
			pW=wrapper.style.width!=''?parseInt(wrapper.style.width):100;
			pH=wrapper.style.height!=''?parseInt(wrapper.style.height):100;
			// normal is up-right
			var LEFT = tpos.left+50;
			var TOP =  tpos.top-pH-10;
			// correct for left
			LEFT=Math.min(LEFT,vW-pW);
			// correct for top
			TOP=Math.max(TOP,pH);
			// move it and show it
			box.setCurrentPos(LEFT,TOP);
			box.showBox();
			// bring to front
			box.moveToFront();
		}
		else{
			console.log("no box match to ",target.id);
		}
	},

	//----------------------------
	// take it away 
	// not those which must be removed by close, kill
	hideBox:function (e){
		if(popManager.timer!=null){
			clearTimeout(popManager.timer);
			popManager.timer=null;
		}
		var target=e.target;
		var boxid=popManager.findMatch(target.id);
		box=itemHolder.getBoxById(boxid);
		// box with kill will live until we kill it
		if(!box.getKill())
			box.hideBox();
	}
}
